/***********************************************************************/
/*                                                                     */
/* Simple sample program to get and print messages of format NBALANCE. */
/* We will read the queue until empty, then exit.                      */
/* Copyright IBM Corp 1997.                                            */
/*                                                                     */
/***********************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <cmqc.h>
#include "NBalance.h"

#define MAX_TEXT 72

enum rc_e { good_rc, bad_rc };

int main( int argc, char *argv[] )
{

 static MQCHAR48   qmname = { '\0' };           /* Queue manager name   */
 MQHCONN           hconn;                       /* Connection handle    */
 MQHOBJ            hq;                          /* Object handle        */
 MQLONG            cc;                          /* Completion code      */
 MQLONG            rc;                          /* Reason               */
 MQLONG            msglen;                      /* Message length       */
 char              msg[ MAX_TEXT ];             /* Error Message        */
 MQOD              mqod  = { MQOD_DEFAULT };    /* Object descriptor    */
 MQMD              mqmd  = { MQMD_DEFAULT };    /* Message descriptor   */
 MQGMO             mqgmo = { MQGMO_DEFAULT };   /* Get options          */
 int               MsgNo;                       /* Messages received    */
 struct NBALANCE   NBalance;                    /* MQ message           */
 int               i;                           /* Index into NBalance  */
 char              AccountHolder[ ACCOUNTHOLDERLEN + 1];  /* As C string*/ 
 char              AccountNumber[ ACCOUNTNUMBERLEN + 1];  /* As C string*/

 if ( argc != 2 )
 {
  sprintf( msg, "Usage: %s queue_name\n", argv[0] );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }
 strcpy( mqod.ObjectName, argv[ 1 ] );

 MQCONN( qmname, &hconn, &cc, &rc );   /* Connect to default QMgr */
 if ( cc != MQCC_OK )
 {
  sprintf( msg, "MQCONN failed - reason code %li\n", rc );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }

 MQOPEN( hconn,
         &mqod,
         MQOO_INPUT_SHARED,
         &hq,
         &cc,
         &rc );
 if ( cc != MQCC_OK )
 {
  sprintf( msg, "MQOPEN failed - reason code %li\n", rc );
  fputs(   msg, stderr );
  exit(    bad_rc );
 }

 mqgmo.Options      |= MQGMO_CONVERT;
 mqgmo.WaitInterval  = MQWI_UNLIMITED;

 for ( MsgNo = 1 ; ; MsgNo++ )     /* Get until queue is empty */
 {

  memcpy( mqmd.MsgId,    MQMI_NONE, sizeof( mqmd.MsgId ) );
  memcpy( mqmd.CorrelId, MQCI_NONE, sizeof( mqmd.CorrelId ) );

  MQGET(  hconn,
          hq,
          &mqmd,
          &mqgmo,
          sizeof( NBalance ),
         &NBalance,
          &msglen,
          &cc,
          &rc );
  if ( cc != MQCC_OK )
  {
   if ( rc == MQRC_NO_MSG_AVAILABLE ) 
   {
    MsgNo--;
    break;
   }
   sprintf( msg, "MQGET failed - reason code %li\n", rc );
   fputs(   msg, stderr );
   exit(    bad_rc );
  }
  printf( "Message received - contains %d balances\n", NBalance.Count );
  for ( i = 0 ; i < NBalance.Count ; i++ )
  {
   printf( " Balance %d\n", i + 1 );
   CharsToCStr( AccountHolder, NBalance.Balance[i].AccountHolder, 
                ACCOUNTHOLDERLEN );
   CharsToCStr( AccountNumber, NBalance.Balance[i].AccountNumber, 
                ACCOUNTNUMBERLEN );
   printf( "   AccountHolder: %s\n", AccountHolder );
   printf( "   AccountNumber: %s\n", AccountNumber );
   printf( "   CurrentBalance: %d\n", 
           NBalance.Balance[i].CurrentBalance );
  }
  printf( "\n" );
 }   /* for loop */

 printf( "%d messages were received - now exiting\n", MsgNo );
 MQCLOSE( hconn,
          &hq,
          MQCO_NONE,
          &cc,
          &rc );
 MQDISC( &hconn, &cc, &rc );
 exit( good_rc );

}  /* main */
